@@ -9,6 +9,8 @@ var searchData=
   ['tapping_2edox',['tapping.dox',['../tapping_8dox.html',1,'']]],
   ['test_2dsuite_2edox',['test-suite.dox',['../test-suite_8dox.html',1,'']]],
   ['tools_2edox',['tools.dox',['../tools_8dox.html',1,'']]],
+  ['touchpad_2djumping_2dcursors_2edox',['touchpad-jumping-cursors.dox',['../touchpad-jumping-cursors_8dox.html',1,'']]],
+  ['touchpad_20jumping_20cursor_20bugs',['Touchpad jumping cursor bugs',['../touchpad_jumping_cursor.html',1,'touchpads']]],
   ['touchpads',['Touchpads',['../touchpads.html',1,'']]],
   ['touchpads_2edox',['touchpads.dox',['../touchpads_8dox.html',1,'']]],
   ['touchscreens',['Touchscreens',['../touchscreens.html',1,'']]]
