@@ -1,7 +1,7 @@
 var searchData=
 [
   ['libinput_5fbutton_5fstate',['libinput_button_state',['../group__device.html#ga543a8bb3f2d7421f76e63bb4ee2fb5bf',1,'libinput.h']]],
-  ['libinput_5fconfig_5faccel_5fprofile',['libinput_config_accel_profile',['../libinput_8h.html#ad63796972347f318b180e322e35cee79',1,'libinput.h']]],
+  ['libinput_5fconfig_5faccel_5fprofile',['libinput_config_accel_profile',['../group__config.html#gad63796972347f318b180e322e35cee79',1,'libinput.h']]],
   ['libinput_5fconfig_5fclick_5fmethod',['libinput_config_click_method',['../group__config.html#ga3e5dc19a83752430d5e8be7fb2020663',1,'libinput.h']]],
   ['libinput_5fconfig_5fdrag_5flock_5fstate',['libinput_config_drag_lock_state',['../group__config.html#gaedaabeb8e33f58a032c2e099b91f4579',1,'libinput.h']]],
   ['libinput_5fconfig_5fdrag_5fstate',['libinput_config_drag_state',['../group__config.html#gaddca1d98dbf1b53fc93de3d323588423',1,'libinput.h']]],
@@ -18,6 +18,8 @@ var searchData=
   ['libinput_5flog_5fpriority',['libinput_log_priority',['../group__base.html#gaa7e9ebbd1a13fe2ed6f44e69df3216e1',1,'libinput.h']]],
   ['libinput_5fpointer_5faxis',['libinput_pointer_axis',['../group__device.html#ga82cd15b9476a4a60f25470df8d67483c',1,'libinput.h']]],
   ['libinput_5fpointer_5faxis_5fsource',['libinput_pointer_axis_source',['../group__device.html#ga76c012d8f6d7656fb795dc7bdf9d6551',1,'libinput.h']]],
+  ['libinput_5ftablet_5fpad_5fring_5faxis_5fsource',['libinput_tablet_pad_ring_axis_source',['../group__event__tablet.html#gafb0259f1d3f79e4ecaa74eb307186473',1,'libinput.h']]],
+  ['libinput_5ftablet_5fpad_5fstrip_5faxis_5fsource',['libinput_tablet_pad_strip_axis_source',['../group__event__tablet.html#ga09790bd108d3a288bb387aeb3a9a2dd8',1,'libinput.h']]],
   ['libinput_5ftablet_5ftool_5fproximity_5fstate',['libinput_tablet_tool_proximity_state',['../group__device.html#ga54f9b346c0338de742583a8e7c2b0628',1,'libinput.h']]],
   ['libinput_5ftablet_5ftool_5ftip_5fstate',['libinput_tablet_tool_tip_state',['../group__device.html#gabc8383829ae67efea5543fb8af117091',1,'libinput.h']]],
   ['libinput_5ftablet_5ftool_5ftype',['libinput_tablet_tool_type',['../group__device.html#ga8348e628baa2646d31abf0fe7245924f',1,'libinput.h']]]
