@@ -36,19 +36,6 @@
 #define THUMB_MOVE_TIMEOUT ms2us(300)
 #define FAKE_FINGER_OVERFLOW (1 << 7)
 
-static inline int
-tp_hysteresis(int in, int center, int margin)
-{
-	int diff = in - center;
-	if (abs(diff) <= margin)
-		return center;
-
-	if (diff > margin)
-		return center + diff - margin;
-	else
-		return center + diff + margin;
-}
-
 static inline struct device_coords *
 tp_motion_history_offset(struct tp_touch *t, int offset)
 {
@@ -105,12 +92,12 @@ tp_motion_hysteresis(struct tp_dispatch *tp,
 	if (t->history.count == 0) {
 		t->hysteresis_center = t->point;
 	} else {
-		x = tp_hysteresis(x,
-				  t->hysteresis_center.x,
-				  tp->hysteresis_margin.x);
-		y = tp_hysteresis(y,
-				  t->hysteresis_center.y,
-				  tp->hysteresis_margin.y);
+		x = evdev_hysteresis(x,
+				     t->hysteresis_center.x,
+				     tp->hysteresis_margin.x);
+		y = evdev_hysteresis(y,
+				     t->hysteresis_center.y,
+				     tp->hysteresis_margin.y);
 		t->hysteresis_center.x = x;
 		t->hysteresis_center.y = y;
 		t->point.x = x;
@@ -335,6 +322,7 @@ tp_process_absolute(struct tp_dispatch *tp,
 			tp_end_sequence(tp, t, time);
 		break;
 	case ABS_MT_PRESSURE:
+		t->pressure_delta = e->value - t->pressure;
 		t->pressure = e->value;
 		t->dirty = true;
 		tp->queued |= TOUCHPAD_EVENT_OTHERAXIS;
@@ -880,7 +868,7 @@ tp_position_fake_touches(struct tp_dispatch *tp)
 }
 
 static inline bool
-tp_need_motion_history_reset(struct tp_dispatch *tp, uint64_t time)
+tp_need_motion_history_reset(struct tp_dispatch *tp)
 {
 	bool rc = false;
 
@@ -918,6 +906,25 @@ tp_need_motion_history_reset(struct tp_dispatch *tp, uint64_t time)
 	return rc;
 }
 
+static bool
+tp_detect_jumps(const struct tp_dispatch *tp, struct tp_touch *t)
+{
+	struct device_coords *last;
+	double dx, dy;
+	const int JUMP_THRESHOLD_MM = 20;
+
+	if (t->history.count == 0)
+		return false;
+
+	/* called before tp_motion_history_push, so offset 0 is the most
+	 * recent coordinate */
+	last = tp_motion_history_offset(t, 0);
+	dx = fabs(t->point.x - last->x) / tp->device->abs.absinfo_x->resolution;
+	dy = fabs(t->point.y - last->y) / tp->device->abs.absinfo_y->resolution;
+
+	return hypot(dx, dy) > JUMP_THRESHOLD_MM;
+}
+
 static void
 tp_process_state(struct tp_dispatch *tp, uint64_t time)
 {
@@ -930,7 +937,7 @@ tp_process_state(struct tp_dispatch *tp, uint64_t time)
 	tp_unhover_touches(tp, time);
 	tp_position_fake_touches(tp);
 
-	want_motion_reset = tp_need_motion_history_reset(tp, time);
+	want_motion_reset = tp_need_motion_history_reset(tp);
 
 	for (i = 0; i < tp->ntouches; i++) {
 		t = tp_get_touch(tp, i);
@@ -946,6 +953,17 @@ tp_process_state(struct tp_dispatch *tp, uint64_t time)
 		if (!t->dirty)
 			continue;
 
+		if (t->pressure_delta < -7)
+			tp_motion_history_reset(t);
+
+		if (tp_detect_jumps(tp, t)) {
+			log_bug_kernel(tp_libinput_context(tp),
+				       "Touch jump detected and discarded.\n"
+				       "See %stouchpad_jumping_cursor for details\n",
+				       HTTP_DOC_LINK);
+			tp_motion_history_reset(t);
+		}
+
 		tp_thumb_detect(tp, t, time);
 		tp_palm_detect(tp, t, time);
 
@@ -1344,6 +1362,10 @@ tp_want_dwt(struct evdev_device *touchpad,
 	if (bus_tp == BUS_I8042 && bus_kbd != bus_tp)
 		return false;
 
+	/* Logitech does not have internal touchpads */
+	if (vendor_tp == VENDOR_ID_LOGITECH)
+		return false;
+
 	/* For Apple touchpads, always use its internal keyboard */
 	if (vendor_tp == VENDOR_ID_APPLE) {
 		return vendor_kbd == vendor_tp &&
@@ -2019,6 +2041,9 @@ tp_init_hysteresis(struct tp_dispatch *tp)
 	    (tp->device->model_flags & EVDEV_MODEL_SYNAPTICS_SERIAL_TOUCHPAD))
 		goto want_hysteresis;
 
+	if (tp->device->model_flags & EVDEV_MODEL_WOBBLY_TOUCHPAD)
+		goto want_hysteresis;
+
 	tp->hysteresis_margin.x = 0;
 	tp->hysteresis_margin.y = 0;
 
