@@ -43,6 +43,10 @@
 #include "filter.h"
 #include "libinput-private.h"
 
+#if HAVE_LIBWACOM
+#include <libwacom/libwacom.h>
+#endif
+
 #define DEFAULT_WHEEL_CLICK_ANGLE 15
 #define DEFAULT_MIDDLE_BUTTON_SCROLL_TIMEOUT ms2us(200)
 
@@ -61,7 +65,7 @@ enum evdev_device_udev_tags {
         EVDEV_UDEV_TAG_TABLET = (1 << 5),
         EVDEV_UDEV_TAG_JOYSTICK = (1 << 6),
         EVDEV_UDEV_TAG_ACCELEROMETER = (1 << 7),
-        EVDEV_UDEV_TAG_BUTTONSET = (1 << 8),
+        EVDEV_UDEV_TAG_TABLET_PAD = (1 << 8),
         EVDEV_UDEV_TAG_POINTINGSTICK = (1 << 9),
 };
 
@@ -78,7 +82,7 @@ static const struct evdev_udev_tag_match evdev_udev_tag_matches[] = {
 	{"ID_INPUT_TOUCHPAD",		EVDEV_UDEV_TAG_TOUCHPAD},
 	{"ID_INPUT_TOUCHSCREEN",	EVDEV_UDEV_TAG_TOUCHSCREEN},
 	{"ID_INPUT_TABLET",		EVDEV_UDEV_TAG_TABLET},
-	{"ID_INPUT_TABLET_PAD",		EVDEV_UDEV_TAG_BUTTONSET},
+	{"ID_INPUT_TABLET_PAD",		EVDEV_UDEV_TAG_TABLET_PAD},
 	{"ID_INPUT_JOYSTICK",		EVDEV_UDEV_TAG_JOYSTICK},
 	{"ID_INPUT_ACCELEROMETER",	EVDEV_UDEV_TAG_ACCELEROMETER},
 	{"ID_INPUT_POINTINGSTICK",	EVDEV_UDEV_TAG_POINTINGSTICK},
@@ -154,14 +158,17 @@ evdev_pointer_notify_physical_button(struct evdev_device *device,
 					     state))
 			return;
 
-	evdev_pointer_notify_button(device, time, button, state);
+	evdev_pointer_notify_button(device,
+				    time,
+				    (unsigned int)button,
+				    state);
 }
 
-void
-evdev_pointer_notify_button(struct evdev_device *device,
-			    uint64_t time,
-			    int button,
-			    enum libinput_button_state state)
+static void
+evdev_pointer_post_button(struct evdev_device *device,
+			  uint64_t time,
+			  unsigned int button,
+			  enum libinput_button_state state)
 {
 	int down_count;
 
@@ -182,6 +189,59 @@ evdev_pointer_notify_button(struct evdev_device *device,
 
 }
 
+static void
+evdev_button_scroll_timeout(uint64_t time, void *data)
+{
+	struct evdev_device *device = data;
+
+	device->scroll.button_scroll_active = true;
+}
+
+static void
+evdev_button_scroll_button(struct evdev_device *device,
+			   uint64_t time, int is_press)
+{
+	device->scroll.button_scroll_btn_pressed = is_press;
+
+	if (is_press) {
+		libinput_timer_set(&device->scroll.timer,
+				   time + DEFAULT_MIDDLE_BUTTON_SCROLL_TIMEOUT);
+		device->scroll.button_down_time = time;
+	} else {
+		libinput_timer_cancel(&device->scroll.timer);
+		if (device->scroll.button_scroll_active) {
+			evdev_stop_scroll(device, time,
+					  LIBINPUT_POINTER_AXIS_SOURCE_CONTINUOUS);
+			device->scroll.button_scroll_active = false;
+		} else {
+			/* If the button is released quickly enough emit the
+			 * button press/release events. */
+			evdev_pointer_post_button(device,
+					device->scroll.button_down_time,
+					device->scroll.button,
+					LIBINPUT_BUTTON_STATE_PRESSED);
+			evdev_pointer_post_button(device, time,
+					device->scroll.button,
+					LIBINPUT_BUTTON_STATE_RELEASED);
+		}
+	}
+}
+
+void
+evdev_pointer_notify_button(struct evdev_device *device,
+			    uint64_t time,
+			    unsigned int button,
+			    enum libinput_button_state state)
+{
+	if (device->scroll.method == LIBINPUT_CONFIG_SCROLL_ON_BUTTON_DOWN &&
+	    button == device->scroll.button) {
+		evdev_button_scroll_button(device, time, state);
+		return;
+	}
+
+	evdev_pointer_post_button(device, time, button, state);
+}
+
 void
 evdev_device_led_update(struct evdev_device *device, enum libinput_led leds)
 {
@@ -273,30 +333,59 @@ evdev_post_trackpoint_scroll(struct evdev_device *device,
 			     uint64_t time)
 {
 	if (device->scroll.method != LIBINPUT_CONFIG_SCROLL_ON_BUTTON_DOWN ||
-	    !hw_is_key_down(device, device->scroll.button))
+	    !device->scroll.button_scroll_btn_pressed)
 		return false;
 
 	if (device->scroll.button_scroll_active)
 		evdev_post_scroll(device, time,
 				  LIBINPUT_POINTER_AXIS_SOURCE_CONTINUOUS,
 				  &unaccel);
+	/* if the button is down but scroll is not active, we're within the
+	   timeout where swallow motion events but don't post scroll buttons */
 
 	return true;
 }
 
+static inline bool
+evdev_filter_defuzz_touch(struct evdev_device *device, struct mt_slot *slot)
+{
+	struct device_coords point;
+
+	if (!device->mt.want_hysteresis)
+		return false;
+
+	point.x = evdev_hysteresis(slot->point.x,
+				   slot->hysteresis_center.x,
+				   device->mt.hysteresis_margin.x);
+	point.y = evdev_hysteresis(slot->point.y,
+				   slot->hysteresis_center.y,
+				   device->mt.hysteresis_margin.y);
+
+	slot->hysteresis_center = slot->point;
+	if (point.x == slot->point.x && point.y == slot->point.y)
+		return true;
+
+	slot->point = point;
+
+	return false;
+}
+
 static void
 evdev_flush_pending_event(struct evdev_device *device, uint64_t time)
 {
 	struct libinput *libinput = device->base.seat->libinput;
-	int slot;
+	int slot_idx;
 	int seat_slot;
 	struct libinput_device *base = &device->base;
 	struct libinput_seat *seat = base->seat;
 	struct normalized_coords accel, unaccel;
 	struct device_coords point;
 	struct device_float_coords raw;
+	struct mt_slot *slot = NULL;
 
-	slot = device->mt.slot;
+	slot_idx = device->mt.slot;
+	if (device->mt.slots)
+		slot = &device->mt.slots[slot_idx];
 
 	switch (device->pending_event) {
 	case EVDEV_NONE:
@@ -337,7 +426,7 @@ evdev_flush_pending_event(struct evdev_device *device, uint64_t time)
 		if (!(device->seat_caps & EVDEV_DEVICE_TOUCH))
 			break;
 
-		if (device->mt.slots[slot].seat_slot != -1) {
+		if (slot->seat_slot != -1) {
 			log_bug_kernel(libinput,
 				       "%s: Driver sent multiple touch down for the "
 				       "same slot",
@@ -346,45 +435,49 @@ evdev_flush_pending_event(struct evdev_device *device, uint64_t time)
 		}
 
 		seat_slot = ffs(~seat->slot_map) - 1;
-		device->mt.slots[slot].seat_slot = seat_slot;
+		slot->seat_slot = seat_slot;
 
 		if (seat_slot == -1)
 			break;
 
 		seat->slot_map |= 1 << seat_slot;
-		point = device->mt.slots[slot].point;
+		point = slot->point;
+		slot->hysteresis_center = point;
 		evdev_transform_absolute(device, &point);
 
-		touch_notify_touch_down(base, time, slot, seat_slot,
+		touch_notify_touch_down(base, time, slot_idx, seat_slot,
 					&point);
 		break;
 	case EVDEV_ABSOLUTE_MT_MOTION:
 		if (!(device->seat_caps & EVDEV_DEVICE_TOUCH))
 			break;
 
-		seat_slot = device->mt.slots[slot].seat_slot;
-		point = device->mt.slots[slot].point;
+		seat_slot = slot->seat_slot;
+		point = slot->point;
 
 		if (seat_slot == -1)
 			break;
 
+		if (evdev_filter_defuzz_touch(device, slot))
+			break;
+
 		evdev_transform_absolute(device, &point);
-		touch_notify_touch_motion(base, time, slot, seat_slot,
+		touch_notify_touch_motion(base, time, slot_idx, seat_slot,
 					  &point);
 		break;
 	case EVDEV_ABSOLUTE_MT_UP:
 		if (!(device->seat_caps & EVDEV_DEVICE_TOUCH))
 			break;
 
-		seat_slot = device->mt.slots[slot].seat_slot;
-		device->mt.slots[slot].seat_slot = -1;
+		seat_slot = slot->seat_slot;
+		slot->seat_slot = -1;
 
 		if (seat_slot == -1)
 			break;
 
 		seat->slot_map &= ~(1 << seat_slot);
 
-		touch_notify_touch_up(base, time, slot, seat_slot);
+		touch_notify_touch_up(base, time, slot_idx, seat_slot);
 		break;
 	case EVDEV_ABSOLUTE_TOUCH_DOWN:
 		if (!(device->seat_caps & EVDEV_DEVICE_TOUCH))
@@ -480,42 +573,6 @@ get_key_type(uint16_t code)
 	return EVDEV_KEY_TYPE_NONE;
 }
 
-static void
-evdev_button_scroll_timeout(uint64_t time, void *data)
-{
-	struct evdev_device *device = data;
-
-	device->scroll.button_scroll_active = true;
-}
-
-static void
-evdev_button_scroll_button(struct evdev_device *device,
-			   uint64_t time, int is_press)
-{
-	if (is_press) {
-		libinput_timer_set(&device->scroll.timer,
-				   time + DEFAULT_MIDDLE_BUTTON_SCROLL_TIMEOUT);
-		device->scroll.button_down_time = time;
-	} else {
-		libinput_timer_cancel(&device->scroll.timer);
-		if (device->scroll.button_scroll_active) {
-			evdev_stop_scroll(device, time,
-					  LIBINPUT_POINTER_AXIS_SOURCE_CONTINUOUS);
-			device->scroll.button_scroll_active = false;
-		} else {
-			/* If the button is released quickly enough emit the
-			 * button press/release events. */
-			evdev_pointer_notify_physical_button(device,
-					device->scroll.button_down_time,
-					device->scroll.button,
-					LIBINPUT_BUTTON_STATE_PRESSED);
-			evdev_pointer_notify_physical_button(device, time,
-					device->scroll.button,
-					LIBINPUT_BUTTON_STATE_RELEASED);
-		}
-	}
-}
-
 static void
 evdev_process_touch_button(struct evdev_device *device,
 			   uint64_t time, int value)
@@ -576,11 +633,6 @@ evdev_process_key(struct evdev_device *device,
 				   LIBINPUT_KEY_STATE_RELEASED);
 		break;
 	case EVDEV_KEY_TYPE_BUTTON:
-		if (device->scroll.method == LIBINPUT_CONFIG_SCROLL_ON_BUTTON_DOWN &&
-		    e->code == device->scroll.button) {
-			evdev_button_scroll_button(device, time, e->value);
-			break;
-		}
 		evdev_pointer_notify_physical_button(
 			device,
 			time,
@@ -1681,6 +1733,7 @@ evdev_read_model_flags(struct evdev_device *device)
 		{ "LIBINPUT_MODEL_CYAPA", EVDEV_MODEL_CYAPA },
 		{ "LIBINPUT_MODEL_ALPS_RUSHMORE", EVDEV_MODEL_ALPS_RUSHMORE },
 		{ "LIBINPUT_MODEL_LENOVO_T450_TOUCHPAD", EVDEV_MODEL_LENOVO_T450_TOUCHPAD },
+		{ "LIBINPUT_MODEL_WOBBLY_TOUCHPAD", EVDEV_MODEL_WOBBLY_TOUCHPAD },
 		{ NULL, EVDEV_MODEL_DEFAULT },
 	};
 	const struct model_map *m = model_map;
@@ -1688,8 +1741,13 @@ evdev_read_model_flags(struct evdev_device *device)
 
 	while (m->property) {
 		if (!!udev_device_get_property_value(device->udev_device,
-						     m->property))
+						     m->property)) {
+			log_debug(device->base.seat->libinput,
+				  "%s: tagged as %s\n",
+				  evdev_device_get_sysname(device),
+				  m->property);
 			model_flags |= m->model;
+		}
 		m++;
 	}
 
@@ -1994,6 +2052,12 @@ evdev_configure_mt_device(struct evdev_device *device)
 	device->mt.slots_len = num_slots;
 	device->mt.slot = active_slot;
 
+	if (device->abs.absinfo_x->fuzz || device->abs.absinfo_y->fuzz) {
+		device->mt.want_hysteresis = true;
+		device->mt.hysteresis_margin.x = device->abs.absinfo_x->fuzz/2;
+		device->mt.hysteresis_margin.y = device->abs.absinfo_y->fuzz/2;
+	}
+
 	return 0;
 }
 
@@ -2027,7 +2091,7 @@ evdev_configure_device(struct evdev_device *device)
 		 udev_tags & EVDEV_UDEV_TAG_POINTINGSTICK ? " Pointingstick" : "",
 		 udev_tags & EVDEV_UDEV_TAG_JOYSTICK ? " Joystick" : "",
 		 udev_tags & EVDEV_UDEV_TAG_ACCELEROMETER ? " Accelerometer" : "",
-		 udev_tags & EVDEV_UDEV_TAG_BUTTONSET ? " Buttonset" : "");
+		 udev_tags & EVDEV_UDEV_TAG_TABLET_PAD ? " TabletPad" : "");
 
 	if (udev_tags & EVDEV_UDEV_TAG_ACCELEROMETER) {
 		log_info(libinput,
@@ -2045,14 +2109,6 @@ evdev_configure_device(struct evdev_device *device)
 		return -1;
 	}
 
-	/* libwacom assigns tablet _and_ tablet_pad to the pad devices */
-	if (udev_tags & EVDEV_UDEV_TAG_BUTTONSET) {
-		log_info(libinput,
-			 "input device '%s', %s is a buttonset, ignoring\n",
-			 device->devname, devnode);
-		return -1;
-	}
-
 	if (evdev_reject_device(device) == -1) {
 		log_info(libinput,
 			 "input device '%s', %s was rejected.\n",
@@ -2088,7 +2144,17 @@ evdev_configure_device(struct evdev_device *device)
 	tablet_tags = EVDEV_UDEV_TAG_TABLET |
 		      EVDEV_UDEV_TAG_TOUCHPAD |
 		      EVDEV_UDEV_TAG_TOUCHSCREEN;
-	if ((udev_tags & tablet_tags) == EVDEV_UDEV_TAG_TABLET) {
+
+	/* libwacom assigns tablet _and_ tablet_pad to the pad devices */
+	if (udev_tags & EVDEV_UDEV_TAG_TABLET_PAD) {
+		device->dispatch = evdev_tablet_pad_create(device);
+		device->seat_caps |= EVDEV_DEVICE_TABLET_PAD;
+		log_info(libinput,
+			 "input device '%s', %s is a tablet pad\n",
+			 device->devname, devnode);
+		return device->dispatch == NULL ? -1 : 0;
+
+	} else if ((udev_tags & tablet_tags) == EVDEV_UDEV_TAG_TABLET) {
 		device->dispatch = evdev_tablet_create(device);
 		device->seat_caps |= EVDEV_DEVICE_TABLET;
 		log_info(libinput,
@@ -2517,6 +2583,8 @@ evdev_device_has_capability(struct evdev_device *device,
 		return !!(device->seat_caps & EVDEV_DEVICE_GESTURE);
 	case LIBINPUT_DEVICE_CAP_TABLET_TOOL:
 		return !!(device->seat_caps & EVDEV_DEVICE_TABLET);
+	case LIBINPUT_DEVICE_CAP_TABLET_PAD:
+		return !!(device->seat_caps & EVDEV_DEVICE_TABLET_PAD);
 	default:
 		return 0;
 	}
@@ -2858,3 +2926,51 @@ evdev_device_destroy(struct evdev_device *device)
 	free(device->mt.slots);
 	free(device);
 }
+
+bool
+evdev_tablet_has_left_handed(struct evdev_device *device)
+{
+	bool has_left_handed = false;
+#if HAVE_LIBWACOM
+	struct libinput *libinput = device->base.seat->libinput;
+	WacomDeviceDatabase *db;
+	WacomDevice *d = NULL;
+	WacomError *error;
+	const char *devnode;
+
+	db = libwacom_database_new();
+	if (!db) {
+		log_info(libinput,
+			 "Failed to initialize libwacom context.\n");
+		goto out;
+	}
+
+	error = libwacom_error_new();
+	devnode = udev_device_get_devnode(device->udev_device);
+
+	d = libwacom_new_from_path(db,
+				   devnode,
+				   WFALLBACK_NONE,
+				   error);
+
+	if (d) {
+		if (libwacom_is_reversible(d))
+			has_left_handed = true;
+	} else if (libwacom_error_get_code(error) == WERROR_UNKNOWN_MODEL) {
+		log_info(libinput, "Tablet unknown to libwacom\n");
+	} else {
+		log_error(libinput,
+			  "libwacom error: %s\n",
+			  libwacom_error_get_message(error));
+	}
+
+	if (error)
+		libwacom_error_free(&error);
+	if (d)
+		libwacom_destroy(d);
+	libwacom_database_destroy(db);
+
+out:
+#endif
+	return has_left_handed;
+}
