@@ -379,6 +379,9 @@ extern struct litest_test_device litest_cyborg_rat_device;
 extern struct litest_test_device litest_yubikey_device;
 extern struct litest_test_device litest_synaptics_i2c_device;
 extern struct litest_test_device litest_wacom_cintiq_24hd_device;
+extern struct litest_test_device litest_multitouch_fuzz_screen_device;
+extern struct litest_test_device litest_wacom_intuos3_pad_device;
+extern struct litest_test_device litest_wacom_intuos5_pad_device;
 
 struct litest_test_device* devices[] = {
 	&litest_synaptics_clickpad_device,
@@ -424,6 +427,9 @@ struct litest_test_device* devices[] = {
 	&litest_yubikey_device,
 	&litest_synaptics_i2c_device,
 	&litest_wacom_cintiq_24hd_device,
+	&litest_multitouch_fuzz_screen_device,
+	&litest_wacom_intuos3_pad_device,
+	&litest_wacom_intuos5_pad_device,
 	NULL,
 };
 
@@ -1494,23 +1500,39 @@ litest_touch_move_extended(struct litest_device *d,
 }
 
 void
-litest_touch_move_to(struct litest_device *d,
-		     unsigned int slot,
-		     double x_from, double y_from,
-		     double x_to, double y_to,
-		     int steps, int sleep_ms)
+litest_touch_move_to_extended(struct litest_device *d,
+			      unsigned int slot,
+			      double x_from, double y_from,
+			      double x_to, double y_to,
+			      struct axis_replacement *axes,
+			      int steps, int sleep_ms)
 {
 	for (int i = 0; i < steps - 1; i++) {
-		litest_touch_move(d, slot,
-				  x_from + (x_to - x_from)/steps * i,
-				  y_from + (y_to - y_from)/steps * i);
+		litest_touch_move_extended(d, slot,
+					   x_from + (x_to - x_from)/steps * i,
+					   y_from + (y_to - y_from)/steps * i,
+					   axes);
 		if (sleep_ms) {
 			libinput_dispatch(d->libinput);
 			msleep(sleep_ms);
 			libinput_dispatch(d->libinput);
 		}
 	}
-	litest_touch_move(d, slot, x_to, y_to);
+	litest_touch_move_extended(d, slot, x_to, y_to, axes);
+}
+
+void
+litest_touch_move_to(struct litest_device *d,
+		     unsigned int slot,
+		     double x_from, double y_from,
+		     double x_to, double y_to,
+		     int steps, int sleep_ms)
+{
+	litest_touch_move_to_extended(d, slot,
+				      x_from, y_from,
+				      x_to, y_to,
+				      NULL,
+				      steps, sleep_ms);
 }
 
 static int
@@ -1793,6 +1815,15 @@ litest_scale_axis(const struct litest_device *d,
 	return (abs->maximum - abs->minimum) * val/100.0 + abs->minimum;
 }
 
+static inline int
+litest_scale_range(int min, int max, double val)
+{
+	litest_assert_int_ge((int)val, 0);
+	litest_assert_int_le((int)val, 100);
+
+	return (max - min) * val/100.0 + min;
+}
+
 int
 litest_scale(const struct litest_device *d, unsigned int axis, double val)
 {
@@ -1803,12 +1834,120 @@ litest_scale(const struct litest_device *d, unsigned int axis, double val)
 	if (axis <= ABS_Y) {
 		min = d->interface->min[axis];
 		max = d->interface->max[axis];
-		return (max - min) * val/100.0 + min;
+
+		return litest_scale_range(min, max, val);
 	} else {
 		return litest_scale_axis(d, axis, val);
 	}
 }
 
+static inline int
+auto_assign_pad_value(struct litest_device *dev,
+		      struct input_event *ev,
+		      double value)
+{
+	const struct input_absinfo *abs;
+
+	if (ev->value != LITEST_AUTO_ASSIGN ||
+	    ev->type != EV_ABS)
+		return value;
+
+	abs = libevdev_get_abs_info(dev->evdev, ev->code);
+	litest_assert_notnull(abs);
+
+	if (ev->code == ABS_RX || ev->code == ABS_RY) {
+		double min = abs->minimum != 0 ? log2(abs->minimum) : 0,
+		       max = abs->maximum != 0 ? log2(abs->maximum) : 0;
+
+		/* Value 0 is reserved for finger up, so a value of 0% is
+		 * actually 1 */
+		if (value == 0.0) {
+			return 1;
+		} else {
+			value = litest_scale_range(min, max, value);
+			return pow(2, value);
+		}
+	} else {
+		return litest_scale_range(abs->minimum, abs->maximum, value);
+	}
+}
+
+void
+litest_pad_ring_start(struct litest_device *d, double value)
+{
+	struct input_event *ev;
+
+	ev = d->interface->pad_ring_start_events;
+	while (ev && (int16_t)ev->type != -1 && (int16_t)ev->code != -1) {
+		value = auto_assign_pad_value(d, ev, value);
+		litest_event(d, ev->type, ev->code, value);
+		ev++;
+	}
+}
+
+void
+litest_pad_ring_change(struct litest_device *d, double value)
+{
+	struct input_event *ev;
+
+	ev = d->interface->pad_ring_change_events;
+	while (ev && (int16_t)ev->type != -1 && (int16_t)ev->code != -1) {
+		value = auto_assign_pad_value(d, ev, value);
+		litest_event(d, ev->type, ev->code, value);
+		ev++;
+	}
+}
+
+void
+litest_pad_ring_end(struct litest_device *d)
+{
+	struct input_event *ev;
+
+	ev = d->interface->pad_ring_end_events;
+	while (ev && (int16_t)ev->type != -1 && (int16_t)ev->code != -1) {
+		litest_event(d, ev->type, ev->code, ev->value);
+		ev++;
+	}
+}
+
+void
+litest_pad_strip_start(struct litest_device *d, double value)
+{
+	struct input_event *ev;
+
+	ev = d->interface->pad_strip_start_events;
+	while (ev && (int16_t)ev->type != -1 && (int16_t)ev->code != -1) {
+		value = auto_assign_pad_value(d, ev, value);
+		litest_event(d, ev->type, ev->code, value);
+		ev++;
+	}
+}
+
+void
+litest_pad_strip_change(struct litest_device *d, double value)
+{
+	struct input_event *ev;
+
+	ev = d->interface->pad_strip_change_events;
+	while (ev && (int16_t)ev->type != -1 && (int16_t)ev->code != -1) {
+		value = auto_assign_pad_value(d, ev, value);
+		litest_event(d, ev->type, ev->code, value);
+		ev++;
+	}
+}
+
+void
+litest_pad_strip_end(struct litest_device *d)
+{
+	struct input_event *ev;
+
+	ev = d->interface->pad_strip_end_events;
+	while (ev && (int16_t)ev->type != -1 && (int16_t)ev->code != -1) {
+		litest_event(d, ev->type, ev->code, ev->value);
+		ev++;
+	}
+}
+
 void
 litest_wait_for_event(struct libinput *li)
 {
@@ -1947,6 +2086,15 @@ litest_event_type_str(struct libinput_event *event)
 	case LIBINPUT_EVENT_TABLET_TOOL_BUTTON:
 		str = "TABLET TOOL BUTTON";
 		break;
+	case LIBINPUT_EVENT_TABLET_PAD_BUTTON:
+		str = "TABLET PAD BUTTON";
+		break;
+	case LIBINPUT_EVENT_TABLET_PAD_RING:
+		str = "TABLET PAD RING";
+		break;
+	case LIBINPUT_EVENT_TABLET_PAD_STRIP:
+		str = "TABLET PAD STRIP";
+		break;
 	}
 	return str;
 }
@@ -1956,6 +2104,7 @@ litest_print_event(struct libinput_event *event)
 {
 	struct libinput_event_pointer *p;
 	struct libinput_event_tablet_tool *t;
+	struct libinput_event_tablet_pad *pad;
 	struct libinput_device *dev;
 	enum libinput_event_type type;
 	double x, y;
@@ -2003,20 +2152,40 @@ litest_print_event(struct libinput_event *event)
 		break;
 	case LIBINPUT_EVENT_TABLET_TOOL_PROXIMITY:
 		t = libinput_event_get_tablet_tool_event(event);
-		fprintf(stderr, "proximity %d\n",
+		fprintf(stderr, "proximity %d",
 			libinput_event_tablet_tool_get_proximity_state(t));
 		break;
 	case LIBINPUT_EVENT_TABLET_TOOL_TIP:
 		t = libinput_event_get_tablet_tool_event(event);
-		fprintf(stderr, "tip %d\n",
+		fprintf(stderr, "tip %d",
 			libinput_event_tablet_tool_get_tip_state(t));
 		break;
 	case LIBINPUT_EVENT_TABLET_TOOL_BUTTON:
 		t = libinput_event_get_tablet_tool_event(event);
-		fprintf(stderr, "button %d state %d\n",
+		fprintf(stderr, "button %d state %d",
 			libinput_event_tablet_tool_get_button(t),
 			libinput_event_tablet_tool_get_button_state(t));
 		break;
+	case LIBINPUT_EVENT_TABLET_PAD_BUTTON:
+		pad = libinput_event_get_tablet_pad_event(event);
+		fprintf(stderr, "button %d state %d",
+			libinput_event_tablet_pad_get_button_number(pad),
+			libinput_event_tablet_pad_get_button_state(pad));
+		break;
+	case LIBINPUT_EVENT_TABLET_PAD_RING:
+		pad = libinput_event_get_tablet_pad_event(event);
+		fprintf(stderr, "ring %d position %.2f source %d",
+			libinput_event_tablet_pad_get_ring_number(pad),
+			libinput_event_tablet_pad_get_ring_position(pad),
+			libinput_event_tablet_pad_get_ring_source(pad));
+		break;
+	case LIBINPUT_EVENT_TABLET_PAD_STRIP:
+		pad = libinput_event_get_tablet_pad_event(event);
+		fprintf(stderr, "strip %d position %.2f source %d",
+			libinput_event_tablet_pad_get_ring_number(pad),
+			libinput_event_tablet_pad_get_ring_position(pad),
+			libinput_event_tablet_pad_get_ring_source(pad));
+		break;
 	default:
 		break;
 	}
@@ -2438,6 +2607,81 @@ void litest_assert_tablet_proximity_event(struct libinput *li,
 	libinput_event_destroy(event);
 }
 
+struct libinput_event_tablet_pad *
+litest_is_pad_button_event(struct libinput_event *event,
+			   unsigned int button,
+			   enum libinput_button_state state)
+{
+	struct libinput_event_tablet_pad *p;
+	enum libinput_event_type type = LIBINPUT_EVENT_TABLET_PAD_BUTTON;
+
+	litest_assert(event != NULL);
+	litest_assert_int_eq(libinput_event_get_type(event), type);
+
+	p = libinput_event_get_tablet_pad_event(event);
+	litest_assert(p != NULL);
+
+	litest_assert_int_eq(libinput_event_tablet_pad_get_button_number(p),
+			     button);
+
+	return p;
+}
+
+struct libinput_event_tablet_pad *
+litest_is_pad_ring_event(struct libinput_event *event,
+			 unsigned int number,
+			 enum libinput_tablet_pad_ring_axis_source source)
+{
+	struct libinput_event_tablet_pad *p;
+	enum libinput_event_type type = LIBINPUT_EVENT_TABLET_PAD_RING;
+
+	litest_assert(event != NULL);
+	litest_assert_int_eq(libinput_event_get_type(event), type);
+	p = libinput_event_get_tablet_pad_event(event);
+
+	litest_assert_int_eq(libinput_event_tablet_pad_get_ring_number(p),
+			     number);
+	litest_assert_int_eq(libinput_event_tablet_pad_get_ring_source(p),
+			     source);
+
+	return p;
+}
+
+struct libinput_event_tablet_pad *
+litest_is_pad_strip_event(struct libinput_event *event,
+			  unsigned int number,
+			  enum libinput_tablet_pad_strip_axis_source source)
+{
+	struct libinput_event_tablet_pad *p;
+	enum libinput_event_type type = LIBINPUT_EVENT_TABLET_PAD_STRIP;
+
+	litest_assert(event != NULL);
+	litest_assert_int_eq(libinput_event_get_type(event), type);
+	p = libinput_event_get_tablet_pad_event(event);
+
+	litest_assert_int_eq(libinput_event_tablet_pad_get_strip_number(p),
+			     number);
+	litest_assert_int_eq(libinput_event_tablet_pad_get_strip_source(p),
+			     source);
+
+	return p;
+}
+
+void
+litest_assert_pad_button_event(struct libinput *li,
+			       unsigned int button,
+			       enum libinput_button_state state)
+{
+	struct libinput_event *event;
+	struct libinput_event_tablet_pad *pev;
+
+	litest_wait_for_event(li);
+	event = libinput_get_event(li);
+
+	pev = litest_is_pad_button_event(event, button, state);
+	libinput_event_destroy(libinput_event_tablet_pad_get_base_event(pev));
+}
+
 void
 litest_assert_scroll(struct libinput *li,
 		     enum libinput_pointer_axis axis,
